﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;

using SII.SPS.Windows.NetCompactFramework.PrintClassLibrary.MobilePrinter;

namespace SiiMpPrinterSample
{
    /// <summary>
    /// DiscoveryForm class
    /// </summary>
    public partial class DiscoveryForm : Form
    {
        public delegate void DiscoveryFinishedHandler();                // The definition of the callback handler delegate at the time of discovery completion
        public delegate void DiscoveryCanceledHandler();                // The definition of the callback handler delegate at the time of discovery cancellation
        public DiscoveryFinishedHandler discoveredFinishedHandler;      // Discovery completed at the time of the call-back handler
        public DiscoveryCanceledHandler discoveredCanceledHandler;      // Discovery cancellation at the time of the call-back handler

        private MPSample Mainform;                                     // Sample class
        private PrinterDiscovery discoverer;                            // PrinterDiscovery class
        private PrinterInterface currentInterface;                      // Current PrinterInterface
        private DiscoveryHandler discoveryHandler;                      // Discovery handler
        private PrinterDiscoveryHandler printerDiscoveryHandler;        // Printer discovery handler

        private static List<DiscoveredPrinter> discoveredPrinterList;   // List of discovered printers

        public Boolean bCancel = false;                                 // Cancel flag
        public Boolean bDiscovering = false;                            // Printer search flag



        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="f">An instance of the sample class</param>
        public DiscoveryForm(MPSample f)
        {
            //bCancel = false;
            // To hold an instance of the sample class.
            Mainform = f;
            InitializeComponent();

            // To create a list of PrinterDiscovery class and discovered the printer.
            discoverer = new PrinterDiscovery();
            discoveredPrinterList = new List<DiscoveredPrinter>();

            // Set each handler.
            printerDiscoveryHandler = new PrinterDiscoveryHandler(this);
            discoveryHandler = printerDiscoveryHandler;

            discoveredFinishedHandler = new DiscoveryFinishedHandler(DiscoveryFinished);
            discoveredCanceledHandler = new DiscoveryCanceledHandler(DiscoveryCanceled);

            // To get the current interface.
            currentInterface = Mainform.GetCurrentInterface();

            // To begin the discovery.
            DiscoverPrinter();

        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~DiscoveryForm()
        {
            Dispose();
        }


        /// <summary>
        /// Getter to get the address of the current interface
        /// </summary>
        public String Address
        {
            get
            {
                if (lsbDevices.SelectedIndex.Equals(-1))
                {
                    // SelectedIndex is the case of -1 returns an empty.
                    return "";
                }
                else
                {

                    DiscoveredPrinter DiscoveredPrinter = (DiscoveredPrinter)discoveredPrinterList[lsbDevices.SelectedIndex];

                    if (currentInterface == PrinterInterface.PRN_IF_TCP)
                    {
                        // In the case of TCP / IP
                        return DiscoveredPrinter.GetIpAddress();
                    }
                    else if (currentInterface == PrinterInterface.PRN_IF_BT)
                    {
                        // In the case of Bluetooth
                        return DiscoveredPrinter.GetBluetoothAddress();
                    }
                    else
                    {
                        // Otherwise returns an empty.
                        return "";
                    }

                }
            }
        }

        /// <summary>
        /// Processing of the "Discover" button is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_Discover_Click(object sender, EventArgs e)
        {
            DiscoverPrinter();
        }

        /// <summary>
        /// Processing of the "Discover" button is pressed
        /// </summary>
        private void DiscoverPrinter()
        {
            // The "Discover" button in the Disable and then the Cancel button to Enable.
            button_Discover.Enabled = false;
            button_Cancel.Enabled = true;

            // Switching the display of the label, to begin the discovery to clear the list of the found printer.
            this.lblMessage.Text = ("Searching for printer...");
            this.lblMessage.Refresh();
            ClearDiscoveryList();
            StartDiscover();
        }

        /// <summary>
        /// To clears the list of the found printer.
        /// </summary>
        private void ClearDiscoveryList()
        {
            this.lsbDevices.Items.Clear();
        }

        /// <summary>
        /// To begin the discovery.
        /// </summary>
        private void StartDiscover()
        {
            //bCancel = false;
            Thread t1 = new Thread(new ThreadStart(StartDiscoveryPrinter));
            t1.IsBackground = true;

            // Start the discovery in a different thread.
            t1.Start();
            //t1.Join();
        }

        /// <summary>
        /// Processing of the cancel button is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_Cancel_Click(object sender, EventArgs e)
        {
            bCancel = true;
            CancelDiscoverPrinter();
        }

        /// <summary>
        /// Processing of the cancel button is pressed
        /// </summary>
        private void CancelDiscoverPrinter()
        {
            // The "Discover" button in the Enable and then the Cancel button to Disable.
            button_Discover.Enabled = true;
            button_Cancel.Enabled = false;

            // Switching the display of the label, and clears the list of the found printer.
            this.lblMessage.Text = ("Discovered printers list.");
            this.lblMessage.Refresh();
            ClearDiscoveryList();

            if (!bDiscovering)
            {
                // If it is not in the printer search will be terminated without the cancellation process.
                return;
            }

            // Do the cancellation process.
            CancelDiscover();
        }

        /// <summary>
        /// Cancellation processing
        /// </summary>
        private void CancelDiscover()
        {
            Thread t2 = new Thread(new ThreadStart(CancelDiscoveryPrinter));
            t2.IsBackground = true;

            // Do the cancellation processing in a separate thread.
            t2.Start();
            //t2.Join();
        }

        /// <summary>
        /// To begin the discovery.
        /// </summary>
        private void StartDiscoveryPrinter()
        {
            // The printer search in the flag to true, and the cancellation flag to false.
            Thread.Sleep(10);       // This Sleep is no meaning in particular.
            bDiscovering = true;
            bCancel = false;

            try
            {
                // To begin the discovery.
                discoverer.StartDiscoveryPrinter(discoveryHandler, currentInterface, 1, 10);
            }
            catch (PrinterException ex)
            {
                // If the printer exception occurs, To display the error code.
                MessageBox.Show(ex.Message + "\nErrorCode = " + ex.GetErrorCode().ToString());
                bDiscovering = false;       // The printer search in the flag is set to false.
            }
            catch (Exception ex)
            {
                // If any other exception occurs, to displays the value of the exception in a message box.
                MessageBox.Show(ex.Message);
                bDiscovering = false;       // The printer search flag is set to false.
            } 
        }

        /// <summary>
        /// Cancellation processing
        /// </summary>
        private void CancelDiscoveryPrinter()
        {
            Thread.Sleep(10);       // This Sleep is no meaning in particular.
            try
            {
                // Cancel the discovery.
                discoverer.CancelDiscoveryPrinter();
            }
            catch (PrinterException ex) 
            {
                // If the printer exception occurs, To display the error code.
                MessageBox.Show(ex.Message + "\nErrorCode = " + ex.GetErrorCode().ToString());

            }
            catch (Exception ex) 
            {
                // If any other exception occurs, to display the value of the exception in a message box.
                MessageBox.Show(ex.Message);
            
            } 
        }

        /// <summary>
        /// Creat a list of the found printer.
        /// </summary>
        public void CreatDiscoveredPrinterList()
        {
            // Switching the display of the label, and clears the list of the found printer.
            lblMessage.Text = ("Discovered printers list.");
            lblMessage.Refresh();
            discoveredPrinterList.Clear();

            // To gets the discovered printer information list.
            discoveredPrinterList = discoverer.GetFoundPrinter();

            if (discoveredPrinterList.Count == 0)
            {
                // If that has not been found
                MessageBox.Show("No found!");
                return;
            }

            // Add those that fall under the category of the current I/F to the list box.
            foreach (DiscoveredPrinter prin in discoveredPrinterList)
            {
                if (currentInterface == PrinterInterface.PRN_IF_TCP)
                {
                    this.lsbDevices.Items.Add(
                        prin.GetPrinterModel() +
                        "(" +
                        prin.GetIpAddress() +
                        ")");
                }
                else if (currentInterface == PrinterInterface.PRN_IF_BT)
                {
                    this.lsbDevices.Items.Add(
                        prin.GetPrinterModel() +
                        "(" +
                        prin.GetBluetoothAddress() +
                        ")");
                }
            }

            // To switches the display of the label.
            lblMessage.Text = ("Discovery finished! Select a printer.");
        }

        /// <summary>
        /// List event handler when the box has been selected
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lsbDevices_SelectedIndexChanged(object sender, EventArgs e)
        {
            // If the search has been a printer in the list box is selected, and then set the result of DialogResult.OK.
            DialogResult = DialogResult.OK;
        }



        /// <summary>
        /// Discovery completion of the callback handler
        /// </summary>
        public void DiscoveryFinished()
        {
            // The "Discover" button to Enable, and a cancel button on the state of Disable.
            button_Discover.Enabled = true;
            button_Cancel.Enabled = false;

            try
            {
                // To build a list of the found printer, and the printer search flag to false.
	            CreatDiscoveredPrinterList();
                bDiscovering = false;
            }
            // If the printer exception occurs, To display the error code.
            catch (PrinterException ex) { MessageBox.Show("ErrorCode = " + ex.GetErrorCode().ToString()); }
            catch (Exception ex) { MessageBox.Show(ex.Message); }            
            
        }

        /// <summary>
        /// Callback handler at the time of cancellation of Discovery
        /// </summary>
        public void DiscoveryCanceled()
        {
            // The "Discover" button to Enable, and a cancel button on the state of Disable.
            button_Discover.Enabled = true;
            button_Cancel.Enabled = false;

            // Cancel flag and the printer search flag to false.
            bCancel = false;
            bDiscovering = false;

            // Switching the display of the label, and displays a message box.
            lblMessage.Text = ("Discovery canceled!");
            MessageBox.Show("DiscoveryCanceled!");
        }

        /// <summary>
        /// Event handler to close the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DiscoveryForm_Closing(object sender, CancelEventArgs e)
        {
            if (bDiscovering)
            {
                // To cancel the case Discovery during printers search.
                CancelDiscover();
            }
        }
    }

    /// <summary>
    /// PrinterDiscoveryHandler class
    /// </summary>
    public class PrinterDiscoveryHandler : DiscoveryHandler
    {
        DiscoveryForm discoveryForm;        // DiscoveryForm class

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="f">DiscoveryForm class</param>
        public PrinterDiscoveryHandler(DiscoveryForm f)
        {
            // To hold an instance of the DiscoveryForm class.
            discoveryForm = f;
        }

        /// <summary>
        /// Discovery completed at the time of the call-back handler
        /// </summary>
        public void DiscoveryFinished()
        {
            try
            {
                // Run the callback handler delegates for the time of discovery is complete.
                discoveryForm.Invoke(discoveryForm.discoveredFinishedHandler);
            }
            // If the printer exception occurs, To display the error code.
            catch (PrinterException ex) { MessageBox.Show("ErrorCode = " + ex.GetErrorCode().ToString()); }
            catch (Exception ex) { MessageBox.Show(ex.Message); } 

        }

        /// <summary>
        /// Discovery cancellation of callback handler
        /// </summary>
        public void DiscoveryCanceled()
        {
            if (!discoveryForm.bCancel)
                return;     // It does not do anything if the cancellation flag is false.

            try
            {
                // Run the delegate of Discovery callback handler at the time of the cancellation.
                discoveryForm.Invoke(discoveryForm.discoveredCanceledHandler);
            }
            // If the printer exception occurs, To display the error code.
            catch (PrinterException ex) { MessageBox.Show("ErrorCode = " + ex.GetErrorCode().ToString()); }
            catch (Exception ex) { MessageBox.Show(ex.Message); } 

        }
    }
}