﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SII.SPS.Windows.NetCompactFramework.PrintClassLibrary.MobilePrinter;

namespace SiiMpPrinterSample
{
    /// <summary>
    /// StatusCallbackForm class
    /// </summary>
    public partial class StatusCallbackForm : Form
    {
        // Definitions such as callback handler of printer status
        public delegate void CallbackHandler(PrinterStatus status);
        public CallbackHandler callbackHandler;

        public StatusCallbackHandler statusCallbackHandler;
        private PrinterStatusCallbackHandler printerStatusCallbackHandler;

        private MPSample Mainform;         // Sample class


        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="f">An instance of the sample class</param>
        public StatusCallbackForm(MPSample f)
        {
            // To hold an instance of the sample class.
            Mainform = f;
            InitializeComponent();

            // Set the callback handler.
            printerStatusCallbackHandler = new PrinterStatusCallbackHandler(this);
            statusCallbackHandler = printerStatusCallbackHandler;

            callbackHandler = new CallbackHandler(StatusChanged);

            // Start a call back.
            StartCallback();
        }

        /// <summary>
        /// Get the value of the open flag.
        /// </summary>
        /// <returns>The value of the open flag</returns>
        public bool isOpend()
        {
            return Mainform.mPrinterManager.IsOpened;
        }
        
        /// <summary>
        /// Start a call back.
        /// </summary>
        public void StartCallback()
        {
            try
            {
                // Open the PrinterManager.
                Mainform.Open();

                if (Mainform.mPrinterManager.IsOpened)
                {
                    // Set the callback handler of printer status.
                    Mainform.mPrinterManager.StartCallbackFunction(statusCallbackHandler);
                }
            }
            catch (PrinterException err)
            {
                // If the printer exception occurs, To display the error code.
                MessageBox.Show("Error Code = " + err.GetErrorCode().ToString());
                Mainform.Close();
            }
        }

        /// <summary>
        /// Stop the callback.
        /// </summary>
        public void StopCallback()
        {
            try
            {
                if (Mainform.mPrinterManager.IsOpened)
                {
                    // Stop the callback.
                    Mainform.mPrinterManager.StopCallbackFunction();
                }
            }
            catch (PrinterException err)
            {
                // If the printer exception occurs, To display the error code.
                MessageBox.Show("Error Code = " + err.GetErrorCode().ToString());
            }

        }

        /// <summary>
        /// Callback handler to be called when there is a change in the printer status
        /// </summary>
        /// <param name="status">Printer status</param>
        public void StatusChanged(PrinterStatus status)
        {
            try
            {
                if (!status.GetErrOffline())
                {
                    // Online status
                    label_Offline.Text = "Online";
                }
                else
                {
                    // Offline state
                    label_Offline.Text = "Offline";
                }

                // Set the status of the printer to each label.
                label_BatteryErr.Text = status.GetErrBattery().ToString();                  // Battery error
                label_BatteryState.Text = status.GetStateBattery().ToString();              // Remaining battery capacity
                label_FeedSwitch.Text = status.GetStateFeedSwitch().ToString();             // The state of the feed switch
                label_Flash.Text = status.GetStateFlashMemoryRewriting().ToString();        // Flash memory while writing
                label_Hardware.Text = status.GetErrHardware().ToString();                   // Hardware error
                label_HeadTmp.Text = status.GetErrHeadTemperature().ToString();             // Head temperature error
                label_OutPaper.Text = status.GetErrOutOfPaper().ToString();                 // No paper error
                label_PaperFeed.Text = status.GetStatePaperFeed().ToString();               // Paper feed state
                label_Return.Text = status.GetStateReturnWaiting().ToString();              // Recovery waiting state
                label_Volt.Text = status.GetErrVoltage().ToString();                        // Voltage error

                // MP-A40
                if (Mainform.m_PrinterModel == PrinterModel.PRN_MODEL_MP_A40)
                {
                    label_CoverOpen.Text = status.GetErrCoverOpen().ToString();             // Cover open error
                    label_Jam.Text = status.GetErrMarkPaperJam().ToString();                // Mark detection when paper jam error condition
                }
                // MP-B30
                else if (Mainform.m_PrinterModel == PrinterModel.PRN_MODEL_MP_B30)
                {
                    label_CoverOpen.Text = status.GetErrCoverOpen().ToString();             // Cover open error
                }
                // MP-B20
                else{
                    // do nothing
                }

            }
            catch (PrinterException err)
            {
                // If the printer exception occurs, To display the error code.
                MessageBox.Show("Error Code = " + err.GetErrorCode().ToString());
            }
        }

        /// <summary>
        /// Event handler to close the form
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StatusCallbackForm_Closing(object sender, CancelEventArgs e)
        {
            if (Mainform.mPrinterManager.IsOpened)
            {
                // Close the PrinterManager to stop the call back.
                StopCallback();
                Mainform.Close();
            }
        }
    }

    /// <summary>
    /// PrinterStatusCallbackHandler class
    /// </summary>
    public class PrinterStatusCallbackHandler : StatusCallbackHandler
    {
        StatusCallbackForm statusCallbackForm;      // StatusCallbackForm class

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="f"></param>
        public PrinterStatusCallbackHandler(StatusCallbackForm f)
        {
            // To hold an instance of the StatusCallbackForm class.
            statusCallbackForm = f;
        }

        /// <summary>
        /// Callback handler if there is a change in the printer status
        /// </summary>
        /// <param name="status"></param>
        public void StatusChanged(PrinterStatus status)
        {
            // Run the delegate of the callback handler when there is a change in the printer status.
            statusCallbackForm.Invoke(statusCallbackForm.callbackHandler,
                                     new Object[] {status});
        }
    }
}